/*
 * 
 * Copyright (c) 2004 Steve Slaven, All Rights Reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */
package net.hoopajoo.cheeselab3;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import net.hoopajoo.swingx.*;

public class CheeseMapRoomEditor extends JEscDialog implements ActionListener {
    // TODO: Load this list from file (either preferences or separate)
    public static String[] common_overlays = { "boat", "clock", "dark", "food",
            "guild", "herb", "inn", "key", "legendhome", "mail", "mob",
            "mount", "random", "seekspot", "shop", "trail", "treasure",
            "watchtower", "water" };

    JTextField name, overlays, exit_n, exit_s, exit_e, exit_w, exit_u, exit_d,
            door_n, door_s, door_e, door_w, door_u, door_d;

    CheeseRoomData save_room;

    JTextArea edesc, notes;
    JLabel roomid, title;
    JPanel overlay_panel;

    CheeseMapRoomEditor(Frame owner) {
        super( owner );
        final JDialog dialog = this;
        final int rnum_cols = 5;
        final int doorname_cols = 10;
        final int ta_cols = 50;
        final int ta_rows = 10;

        JPanel panel, subpanel;
        JLabel label;
        JButton button;

        setTitle( "Room Editor" );
        
        Container cpane = getContentPane();

        // Create a label that shows what room we are editing
        cpane.add( title = new JLabel( "name ( roomid )" ), BorderLayout.NORTH );
        title.setHorizontalAlignment( SwingConstants.CENTER );
        title.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );

        // Tabbed pane for all room data that can be edited
        JTabbedPane tp = new JTabbedPane();
        cpane.add( tp, BorderLayout.CENTER );

        // Just room description
        panel = new JPanel( new BorderLayout() );
        panel.add( subpanel = new JPanel( new BorderLayout() ),
                BorderLayout.CENTER );
        subpanel.add( new JLabel( "Room description" ), BorderLayout.NORTH );
        subpanel.add(
                new JScrollPane( edesc = new JTextArea( ta_rows, ta_cols ) ),
                BorderLayout.CENTER );
        edesc.setEditable( false );
        tp.add( "Info", panel );

        // Notes
        panel = new JPanel();
        panel.setLayout( new BorderLayout() );
        panel.add(
                new JScrollPane( notes = new JTextArea( ta_rows, ta_cols ) ),
                BorderLayout.CENTER );
        tp.add( "Notes", panel );

        // Overlays panel
        overlay_panel = new JPanel( new GridBagLayout() );
        tp.add( "Overlays", new JScrollPane( overlay_panel ) );

        // Exits panel
        /*
         * We use the awfully complex GridBagLayout to allow the grid to become
         * just as large as components' preferred size because textfields become
         * quite ugly when they are made too tall.
         */
        GridBagConstraints c;
        panel = new JPanel( new GridBagLayout() );
        c = new GridBagConstraints();
        c.gridy = GridBagConstraints.RELATIVE;
        c.anchor = GridBagConstraints.LINE_START;
        c.ipadx = 10; c.ipady = 5;
        // First column
        c.gridx = 0;
        panel.add( label = new JLabel(), c );
        panel.add( label = new JLabel( "North" ), c );
        panel.add( label = new JLabel( "South" ), c );
        panel.add( label = new JLabel( "East" ), c );
        panel.add( label = new JLabel( "West" ), c );
        panel.add( label = new JLabel( "Up" ), c );
        panel.add( label = new JLabel( "Down" ), c );
        // Second column
        c.gridx++;
        panel.add( label = new JLabel( "To" ), c );
        panel.add( exit_n = new JTextField( rnum_cols ), c );
        panel.add( exit_s = new JTextField( rnum_cols ), c );
        panel.add( exit_e = new JTextField( rnum_cols ), c );
        panel.add( exit_w = new JTextField( rnum_cols ), c );
        panel.add( exit_u = new JTextField( rnum_cols ), c );
        panel.add( exit_d = new JTextField( rnum_cols ), c );
        // Thrid column
        c.gridx++;
        panel.add( label = new JLabel( "Door" ) );
        panel.add( door_n = new JTextField( doorname_cols ), c );
        panel.add( door_s = new JTextField( doorname_cols ), c );
        panel.add( door_e = new JTextField( doorname_cols ), c );
        panel.add( door_w = new JTextField( doorname_cols ), c );
        panel.add( door_u = new JTextField( doorname_cols ), c );
        panel.add( door_d = new JTextField( doorname_cols ), c );
        tp.add( "Exits", panel );

        // Advanced panel
        panel = new JPanel( new BorderLayout() );
        tp.add( "Advanced", panel );
        /*
         * panel.add( subpanel = new JPanel( new BorderLayout() ),
         * BorderLayout.NORTH ); subpanel.add( new JLabel ( "Name" ),
         * BorderLayout.NORTH ); subpanel.add( name = new JTextField( ta_cols ),
         * BorderLayout.CENTER ); panel.add( subpanel = new JPanel( new
         * BorderLayout() ), BorderLayout.CENTER ); subpanel.add( new JLabel (
         * "Room description" ), BorderLayout.NORTH ); subpanel.add( new
         * JScrollPane( edesc = new JTextArea( ta_rows, ta_cols ) ),
         * BorderLayout.CENTER ); name.setEditable( false ); edesc.setEditable(
         * false );
         */
        panel.add( subpanel = new JPanel( new BorderLayout() ),
                BorderLayout.NORTH );
        subpanel.add( new JLabel( "Raw overlay string" ), BorderLayout.NORTH );
        subpanel
                .add( overlays = new JTextField( ta_cols ), BorderLayout.CENTER );
        panel = new JPanel();
        panel.add( button = new JButton( "Save" ) );
        button.addActionListener( this );
        button.setActionCommand( "save" );
        panel.add( button = new JButton( "Cancel" ) );
        button.addActionListener( this );
        button.setActionCommand( "cancel" );
        cpane.add( panel, BorderLayout.PAGE_END );

        // Add change listener to handle synchronization between overlay
        // editing controls
        tp.addChangeListener( new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                build_overlay_panel();
            }
        } );

        pack();
    }

    public void actionPerformed(ActionEvent e) {
        String cmd = e.getActionCommand();

        CheeseLab.dmesg( 3, "Woot!" );
        if( cmd.equals( "save" ) )
            CheeseMap.save_edit_room();

        setVisible( false );
        // dispose();
    }

    public void set_room(CheeseRoomData r) {
        save_room = r;

        title.setText( r.name + " ( " + r.roomid + " )" );
        // name.setText( r.name );
        edesc.setText( r.edesc );
        notes.setText( r.notes );

        exit_n.setText( Integer.toString( r.exit_n ) );
        exit_s.setText( Integer.toString( r.exit_s ) );
        exit_e.setText( Integer.toString( r.exit_e ) );
        exit_w.setText( Integer.toString( r.exit_w ) );
        exit_u.setText( Integer.toString( r.exit_u ) );
        exit_d.setText( Integer.toString( r.exit_d ) );

        door_n.setText( r.door_n );
        door_s.setText( r.door_s );
        door_e.setText( r.door_e );
        door_w.setText( r.door_w );
        door_u.setText( r.door_u );
        door_d.setText( r.door_d );

        // overlay types like mob/food/etc
        String otmp = "";
        for (int i = 0; i < r.overlays.length; i++) {
            if( i != 0 )
                otmp += ",";
            otmp += r.overlays[i];
        }
        overlays.setText( otmp );

        build_overlay_panel();

        pack();
    }

    private void build_overlay_panel() {
        String otmp = overlays.getText();

        // FIXME: This shouldn't depend on CheeseMap.canvas object
        overlay_panel.removeAll();
        for (int i = 0; i < common_overlays.length; i++) {
            boolean checked = otmp.indexOf( common_overlays[i] ) == -1 ? false
                    : true;
            JCheckBox cb = new JCheckBox( common_overlays[i], checked );
            JLabel l;
            if( CheeseMap.canvas != null ) {
                l = new JLabel( new ImageIcon( CheeseMap.canvas
                        .get_image( "overlay_" + common_overlays[i], 32, 32 ) ) );
            } else {
                l = new JLabel();
            }

            GridBagConstraints c = new GridBagConstraints();

            c.gridy = i / 3; // Lay components in 3 columns
            c.gridx = GridBagConstraints.RELATIVE;
            c.anchor = GridBagConstraints.LINE_START;

            l.setBorder( BorderFactory
                            .createEtchedBorder( EtchedBorder.RAISED ) );
            cb.setBorder( BorderFactory.createEmptyBorder( 5, 20, 5, 5 ) );
            cb.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JCheckBox src = (JCheckBox)e.getSource();
                    String otmp = overlays.getText();
                    String ovrl = src.getText();
                    if( src.isSelected() ) {
                        if( !otmp.equals( "" ) )
                            otmp += ",";
                        otmp += ovrl;
                    } else {
                        String[] o = otmp.split( "," );
                        otmp = "";
                        for (int i = 0; i < o.length; i++) {
                            if( !ovrl.equals( o[i] ) ) {
                                if( !otmp.equals( "" ) )
                                    otmp += ",";
                                otmp += o[i];
                            }
                        }
                    }
                    overlays.setText( otmp );
                }
            } );

            overlay_panel.add( cb, c );
            overlay_panel.add( l, c );
        }
    }

    public CheeseRoomData get_room() {
        // Load changes in to save_room and return it
        save_room.notes = notes.getText();

        save_room.exit_n = Integer.parseInt( exit_n.getText() );
        save_room.exit_s = Integer.parseInt( exit_s.getText() );
        save_room.exit_e = Integer.parseInt( exit_e.getText() );
        save_room.exit_w = Integer.parseInt( exit_w.getText() );
        save_room.exit_u = Integer.parseInt( exit_u.getText() );
        save_room.exit_d = Integer.parseInt( exit_d.getText() );

        save_room.door_n = door_n.getText();
        save_room.door_s = door_s.getText();
        save_room.door_e = door_e.getText();
        save_room.door_w = door_w.getText();
        save_room.door_u = door_u.getText();
        save_room.door_d = door_d.getText();

        save_room.overlays = overlays.getText().split( "," );

        return (save_room);
    }
}
